@@ -32,6 +32,9 @@ PLATFORMS = [
     Platform.SWITCH,
 ]
 
+CONF_OLD_DISCOVERY = "old_discovery"
+DEFAULT_CONF_OLD_DISCOVERY = False
+
 DATA_FRITZ = "fritz_data"
 
 DSL_CONNECTION: Literal["dsl"] = "dsl"
@@ -43,12 +46,14 @@ DEFAULT_USERNAME = ""
 
 ERROR_AUTH_INVALID = "invalid_auth"
 ERROR_CANNOT_CONNECT = "cannot_connect"
+ERROR_UPNP_NOT_CONFIGURED = "upnp_not_configured"
 ERROR_UNKNOWN = "unknown_error"
 
 FRITZ_SERVICES = "fritz_services"
 SERVICE_REBOOT = "reboot"
 SERVICE_RECONNECT = "reconnect"
 SERVICE_CLEANUP = "cleanup"
+SERVICE_SET_GUEST_WIFI_PW = "set_guest_wifi_password"
 
 SWITCH_TYPE_DEFLECTION = "CallDeflection"
 SWITCH_TYPE_PORTFORWARD = "PortForward"
@@ -63,3 +68,5 @@ FRITZ_EXCEPTIONS = (
     FritzServiceError,
     FritzLookUpError,
 )
+
+WIFI_STANDARD = {1: "2.4Ghz", 2: "5Ghz", 3: "5Ghz", 4: "Guest"}
