@@ -1,4 +1,6 @@
 """Extend the basic Accessory and Bridge functions."""
+from __future__ import annotations
+
 import logging
 
 from pyhap.accessory import Accessory, Bridge
@@ -7,18 +9,15 @@ from pyhap.const import CATEGORY_OTHER
 from pyhap.util import callback as pyhap_callback
 
 from homeassistant.components import cover
-from homeassistant.components.cover import (
-    DEVICE_CLASS_GARAGE,
-    DEVICE_CLASS_GATE,
-    DEVICE_CLASS_WINDOW,
-)
-from homeassistant.components.media_player import DEVICE_CLASS_TV
+from homeassistant.components.media_player import MediaPlayerDeviceClass
 from homeassistant.components.remote import SUPPORT_ACTIVITY
+from homeassistant.components.sensor import SensorDeviceClass
 from homeassistant.const import (
     ATTR_BATTERY_CHARGING,
     ATTR_BATTERY_LEVEL,
     ATTR_DEVICE_CLASS,
     ATTR_ENTITY_ID,
+    ATTR_HW_VERSION,
     ATTR_MANUFACTURER,
     ATTR_MODEL,
     ATTR_SERVICE,
@@ -27,11 +26,6 @@ from homeassistant.const import (
     ATTR_UNIT_OF_MEASUREMENT,
     CONF_NAME,
     CONF_TYPE,
-    DEVICE_CLASS_CO,
-    DEVICE_CLASS_CO2,
-    DEVICE_CLASS_HUMIDITY,
-    DEVICE_CLASS_ILLUMINANCE,
-    DEVICE_CLASS_TEMPERATURE,
     LIGHT_LUX,
     PERCENTAGE,
     STATE_ON,
@@ -52,13 +46,13 @@ from .const import (
     BRIDGE_SERIAL_NUMBER,
     CHAR_BATTERY_LEVEL,
     CHAR_CHARGING_STATE,
+    CHAR_HARDWARE_REVISION,
     CHAR_STATUS_LOW_BATTERY,
     CONF_FEATURE_LIST,
     CONF_LINKED_BATTERY_CHARGING_SENSOR,
     CONF_LINKED_BATTERY_SENSOR,
     CONF_LOW_BATTERY_THRESHOLD,
     DEFAULT_LOW_BATTERY_THRESHOLD,
-    DEVICE_CLASS_PM25,
     DOMAIN,
     EVENT_HOMEKIT_CHANGED,
     HK_CHARGING,
@@ -67,9 +61,9 @@ from .const import (
     MANUFACTURER,
     MAX_MANUFACTURER_LENGTH,
     MAX_MODEL_LENGTH,
-    MAX_NAME_LENGTH,
     MAX_SERIAL_LENGTH,
     MAX_VERSION_LENGTH,
+    SERV_ACCESSORY_INFO,
     SERV_BATTERY_SERVICE,
     SERVICE_HOMEKIT_RESET_ACCESSORY,
     TYPE_FAUCET,
@@ -83,8 +77,9 @@ from .util import (
     accessory_friendly_name,
     async_dismiss_setup_message,
     async_show_setup_message,
+    cleanup_name_for_homekit,
     convert_to_float,
-    format_sw_version,
+    format_version,
     validate_media_player_features,
 )
 
@@ -97,7 +92,7 @@ SWITCH_TYPES = {
     TYPE_SWITCH: "Switch",
     TYPE_VALVE: "Valve",
 }
-TYPES = Registry()
+TYPES: Registry[str, type[HomeAccessory]] = Registry()
 
 
 def get_accessory(hass, driver, state, aid, config):  # noqa: C901
@@ -126,12 +121,13 @@ def get_accessory(hass, driver, state, aid, config):  # noqa: C901
     elif state.domain == "cover":
         device_class = state.attributes.get(ATTR_DEVICE_CLASS)
 
-        if device_class in (DEVICE_CLASS_GARAGE, DEVICE_CLASS_GATE) and features & (
-            cover.SUPPORT_OPEN | cover.SUPPORT_CLOSE
-        ):
+        if device_class in (
+            cover.CoverDeviceClass.GARAGE,
+            cover.CoverDeviceClass.GATE,
+        ) and features & (cover.SUPPORT_OPEN | cover.SUPPORT_CLOSE):
             a_type = "GarageDoorOpener"
         elif (
-            device_class == DEVICE_CLASS_WINDOW
+            device_class == cover.CoverDeviceClass.WINDOW
             and features & cover.SUPPORT_SET_POSITION
         ):
             a_type = "Window"
@@ -161,7 +157,7 @@ def get_accessory(hass, driver, state, aid, config):  # noqa: C901
         device_class = state.attributes.get(ATTR_DEVICE_CLASS)
         feature_list = config.get(CONF_FEATURE_LIST, [])
 
-        if device_class == DEVICE_CLASS_TV:
+        if device_class == MediaPlayerDeviceClass.TV:
             a_type = "TelevisionMediaPlayer"
         elif validate_media_player_features(state, feature_list):
             a_type = "MediaPlayer"
@@ -170,20 +166,23 @@ def get_accessory(hass, driver, state, aid, config):  # noqa: C901
         device_class = state.attributes.get(ATTR_DEVICE_CLASS)
         unit = state.attributes.get(ATTR_UNIT_OF_MEASUREMENT)
 
-        if device_class == DEVICE_CLASS_TEMPERATURE or unit in (
+        if device_class == SensorDeviceClass.TEMPERATURE or unit in (
             TEMP_CELSIUS,
             TEMP_FAHRENHEIT,
         ):
             a_type = "TemperatureSensor"
-        elif device_class == DEVICE_CLASS_HUMIDITY and unit == PERCENTAGE:
+        elif device_class == SensorDeviceClass.HUMIDITY and unit == PERCENTAGE:
             a_type = "HumiditySensor"
-        elif device_class == DEVICE_CLASS_PM25 or DEVICE_CLASS_PM25 in state.entity_id:
+        elif (
+            device_class == SensorDeviceClass.PM25
+            or SensorDeviceClass.PM25 in state.entity_id
+        ):
             a_type = "AirQualitySensor"
-        elif device_class == DEVICE_CLASS_CO:
+        elif device_class == SensorDeviceClass.CO:
             a_type = "CarbonMonoxideSensor"
-        elif device_class == DEVICE_CLASS_CO2 or "co2" in state.entity_id:
+        elif device_class == SensorDeviceClass.CO2 or "co2" in state.entity_id:
             a_type = "CarbonDioxideSensor"
-        elif device_class == DEVICE_CLASS_ILLUMINANCE or unit in ("lm", LIGHT_LUX):
+        elif device_class == SensorDeviceClass.ILLUMINANCE or unit in ("lm", LIGHT_LUX):
             a_type = "LightSensor"
 
     elif state.domain == "switch":
@@ -200,6 +199,7 @@ def get_accessory(hass, driver, state, aid, config):  # noqa: C901
         "automation",
         "button",
         "input_boolean",
+        "input_button",
         "remote",
         "scene",
         "script",
@@ -240,7 +240,11 @@ class HomeAccessory(Accessory):
     ):
         """Initialize a Accessory object."""
         super().__init__(
-            driver=driver, display_name=name[:MAX_NAME_LENGTH], aid=aid, *args, **kwargs
+            driver=driver,
+            display_name=cleanup_name_for_homekit(name),
+            aid=aid,
+            *args,
+            **kwargs,
         )
         self.config = config or {}
         if device_id:
@@ -253,7 +257,7 @@ class HomeAccessory(Accessory):
             domain = split_entity_id(entity_id)[0].replace("_", " ")
 
         if self.config.get(ATTR_MANUFACTURER) is not None:
-            manufacturer = self.config[ATTR_MANUFACTURER]
+            manufacturer = str(self.config[ATTR_MANUFACTURER])
         elif self.config.get(ATTR_INTEGRATION) is not None:
             manufacturer = self.config[ATTR_INTEGRATION].replace("_", " ").title()
         elif domain:
@@ -261,16 +265,19 @@ class HomeAccessory(Accessory):
         else:
             manufacturer = MANUFACTURER
         if self.config.get(ATTR_MODEL) is not None:
-            model = self.config[ATTR_MODEL]
+            model = str(self.config[ATTR_MODEL])
         elif domain:
             model = domain.title()
         else:
             model = MANUFACTURER
         sw_version = None
         if self.config.get(ATTR_SW_VERSION) is not None:
-            sw_version = format_sw_version(self.config[ATTR_SW_VERSION])
+            sw_version = format_version(self.config[ATTR_SW_VERSION])
         if sw_version is None:
-            sw_version = __version__
+            sw_version = format_version(__version__)
+        hw_version = None
+        if self.config.get(ATTR_HW_VERSION) is not None:
+            hw_version = format_version(self.config[ATTR_HW_VERSION])
 
         self.set_info_service(
             manufacturer=manufacturer[:MAX_MANUFACTURER_LENGTH],
@@ -278,6 +285,15 @@ class HomeAccessory(Accessory):
             serial_number=serial_number[:MAX_SERIAL_LENGTH],
             firmware_revision=sw_version[:MAX_VERSION_LENGTH],
         )
+        if hw_version:
+            serv_info = self.get_service(SERV_ACCESSORY_INFO)
+            char = self.driver.loader.get_char(CHAR_HARDWARE_REVISION)
+            serv_info.add_characteristic(char)
+            serv_info.configure_char(
+                CHAR_HARDWARE_REVISION, value=hw_version[:MAX_VERSION_LENGTH]
+            )
+            self.iid_manager.assign(char)
+            char.broker = self
 
         self.category = category
         self.entity_id = entity_id
@@ -518,7 +534,7 @@ class HomeBridge(Bridge):
         """Initialize a Bridge object."""
         super().__init__(driver, name)
         self.set_info_service(
-            firmware_revision=__version__,
+            firmware_revision=format_version(__version__),
             manufacturer=MANUFACTURER,
             model=BRIDGE_MODEL,
             serial_number=BRIDGE_SERIAL_NUMBER,
