@@ -2,10 +2,9 @@
 from __future__ import annotations
 
 import asyncio
-from collections.abc import Iterable
+from collections.abc import Iterable, Mapping
 import logging
-from types import MappingProxyType
-from typing import Any
+from typing import Any, NamedTuple, cast
 
 from homeassistant.const import (
     ATTR_ENTITY_ID,
@@ -31,6 +30,7 @@ from . import (
     ATTR_RGBW_COLOR,
     ATTR_RGBWW_COLOR,
     ATTR_TRANSITION,
+    ATTR_WHITE,
     ATTR_WHITE_VALUE,
     ATTR_XY_COLOR,
     COLOR_MODE_COLOR_TEMP,
@@ -39,6 +39,7 @@ from . import (
     COLOR_MODE_RGBW,
     COLOR_MODE_RGBWW,
     COLOR_MODE_UNKNOWN,
+    COLOR_MODE_WHITE,
     COLOR_MODE_XY,
     DOMAIN,
 )
@@ -69,13 +70,22 @@ COLOR_GROUP = [
     ATTR_KELVIN,
 ]
 
+
+class ColorModeAttr(NamedTuple):
+    """Map service data parameter to state attribute for a color mode."""
+
+    parameter: str
+    state_attr: str
+
+
 COLOR_MODE_TO_ATTRIBUTE = {
-    COLOR_MODE_COLOR_TEMP: ATTR_COLOR_TEMP,
-    COLOR_MODE_HS: ATTR_HS_COLOR,
-    COLOR_MODE_RGB: ATTR_RGB_COLOR,
-    COLOR_MODE_RGBW: ATTR_RGBW_COLOR,
-    COLOR_MODE_RGBWW: ATTR_RGBWW_COLOR,
-    COLOR_MODE_XY: ATTR_XY_COLOR,
+    COLOR_MODE_COLOR_TEMP: ColorModeAttr(ATTR_COLOR_TEMP, ATTR_COLOR_TEMP),
+    COLOR_MODE_HS: ColorModeAttr(ATTR_HS_COLOR, ATTR_HS_COLOR),
+    COLOR_MODE_RGB: ColorModeAttr(ATTR_RGB_COLOR, ATTR_RGB_COLOR),
+    COLOR_MODE_RGBW: ColorModeAttr(ATTR_RGBW_COLOR, ATTR_RGBW_COLOR),
+    COLOR_MODE_RGBWW: ColorModeAttr(ATTR_RGBWW_COLOR, ATTR_RGBWW_COLOR),
+    COLOR_MODE_WHITE: ColorModeAttr(ATTR_WHITE, ATTR_BRIGHTNESS),
+    COLOR_MODE_XY: ColorModeAttr(ATTR_XY_COLOR, ATTR_XY_COLOR),
 }
 
 DEPRECATED_GROUP = [
@@ -93,6 +103,17 @@ DEPRECATION_WARNING = (
 )
 
 
+def _color_mode_same(cur_state: State, state: State) -> bool:
+    """Test if color_mode is same."""
+    cur_color_mode = cur_state.attributes.get(ATTR_COLOR_MODE, COLOR_MODE_UNKNOWN)
+    saved_color_mode = state.attributes.get(ATTR_COLOR_MODE, COLOR_MODE_UNKNOWN)
+
+    # Guard for scenes etc. which where created before color modes were introduced
+    if saved_color_mode == COLOR_MODE_UNKNOWN:
+        return True
+    return cast(bool, cur_color_mode == saved_color_mode)
+
+
 async def _async_reproduce_state(
     hass: HomeAssistant,
     state: State,
@@ -101,9 +122,7 @@ async def _async_reproduce_state(
     reproduce_options: dict[str, Any] | None = None,
 ) -> None:
     """Reproduce a single state."""
-    cur_state = hass.states.get(state.entity_id)
-
-    if cur_state is None:
+    if (cur_state := hass.states.get(state.entity_id)) is None:
         _LOGGER.warning("Unable to find entity %s", state.entity_id)
         return
 
@@ -119,9 +138,13 @@ async def _async_reproduce_state(
         _LOGGER.warning(DEPRECATION_WARNING, deprecated_attrs)
 
     # Return if we are already at the right state.
-    if cur_state.state == state.state and all(
-        check_attr_equal(cur_state.attributes, state.attributes, attr)
-        for attr in ATTR_GROUP + COLOR_GROUP
+    if (
+        cur_state.state == state.state
+        and _color_mode_same(cur_state, state)
+        and all(
+            check_attr_equal(cur_state.attributes, state.attributes, attr)
+            for attr in ATTR_GROUP + COLOR_GROUP
+        )
     ):
         return
 
@@ -144,16 +167,18 @@ async def _async_reproduce_state(
             # Remove deprecated white value if we got a valid color mode
             service_data.pop(ATTR_WHITE_VALUE, None)
             color_mode = state.attributes[ATTR_COLOR_MODE]
-            if color_attr := COLOR_MODE_TO_ATTRIBUTE.get(color_mode):
-                if color_attr not in state.attributes:
+            if color_mode_attr := COLOR_MODE_TO_ATTRIBUTE.get(color_mode):
+                if color_mode_attr.state_attr not in state.attributes:
                     _LOGGER.warning(
                         "Color mode %s specified but attribute %s missing for: %s",
                         color_mode,
-                        color_attr,
+                        color_mode_attr.state_attr,
                         state.entity_id,
                     )
                     return
-                service_data[color_attr] = state.attributes[color_attr]
+                service_data[color_mode_attr.parameter] = state.attributes[
+                    color_mode_attr.state_attr
+                ]
         else:
             # Fall back to Choosing the first color that is specified
             for color_attr in COLOR_GROUP:
@@ -187,8 +212,6 @@ async def async_reproduce_states(
     )
 
 
-def check_attr_equal(
-    attr1: MappingProxyType, attr2: MappingProxyType, attr_str: str
-) -> bool:
+def check_attr_equal(attr1: Mapping, attr2: Mapping, attr_str: str) -> bool:
     """Return true if the given attributes are equal."""
     return attr1.get(attr_str) == attr2.get(attr_str)
