@@ -1,4 +1,6 @@
 """Support for MQTT fans."""
+from __future__ import annotations
+
 import functools
 import logging
 import math
@@ -10,7 +12,6 @@ from homeassistant.components.fan import (
     ATTR_OSCILLATING,
     ATTR_PERCENTAGE,
     ATTR_PRESET_MODE,
-    ATTR_SPEED,
     SPEED_HIGH,
     SPEED_LOW,
     SPEED_MEDIUM,
@@ -19,8 +20,8 @@ from homeassistant.components.fan import (
     SUPPORT_PRESET_MODE,
     SUPPORT_SET_SPEED,
     FanEntity,
-    speed_list_without_preset_modes,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     CONF_NAME,
     CONF_OPTIMISTIC,
@@ -30,31 +31,34 @@ from homeassistant.const import (
 )
 from homeassistant.core import HomeAssistant, callback
 import homeassistant.helpers.config_validation as cv
-from homeassistant.helpers.reload import async_setup_reload_service
-from homeassistant.helpers.typing import ConfigType
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
+from homeassistant.helpers.typing import ConfigType, DiscoveryInfoType
 from homeassistant.util.percentage import (
     int_states_in_range,
-    ordered_list_item_to_percentage,
-    percentage_to_ordered_list_item,
     percentage_to_ranged_value,
     ranged_value_to_percentage,
 )
 
-from . import (
+from . import MqttCommandTemplate, MqttValueTemplate, subscription
+from .. import mqtt
+from .const import (
+    CONF_COMMAND_TEMPLATE,
     CONF_COMMAND_TOPIC,
+    CONF_ENCODING,
     CONF_QOS,
     CONF_RETAIN,
     CONF_STATE_TOPIC,
-    DOMAIN,
-    PLATFORMS,
-    subscription,
+    CONF_STATE_VALUE_TEMPLATE,
+    PAYLOAD_NONE,
 )
-from .. import mqtt
 from .debug_info import log_messages
-from .mixins import MQTT_ENTITY_COMMON_SCHEMA, MqttEntity, async_setup_entry_helper
+from .mixins import (
+    MQTT_ENTITY_COMMON_SCHEMA,
+    MqttEntity,
+    async_setup_entry_helper,
+    async_setup_platform_helper,
+)
 
-CONF_STATE_VALUE_TEMPLATE = "state_value_template"
-CONF_COMMAND_TEMPLATE = "command_template"
 CONF_PERCENTAGE_STATE_TOPIC = "percentage_state_topic"
 CONF_PERCENTAGE_COMMAND_TOPIC = "percentage_command_topic"
 CONF_PERCENTAGE_VALUE_TEMPLATE = "percentage_value_template"
@@ -102,23 +106,12 @@ MQTT_FAN_ATTRIBUTES_BLOCKED = frozenset(
         fan.ATTR_PERCENTAGE,
         fan.ATTR_PRESET_MODE,
         fan.ATTR_PRESET_MODES,
-        fan.ATTR_SPEED_LIST,
-        fan.ATTR_SPEED,
     }
 )
 
 _LOGGER = logging.getLogger(__name__)
 
 
-def valid_fan_speed_configuration(config):
-    """Validate that the fan speed configuration is valid, throws if it isn't."""
-    if config.get(CONF_SPEED_COMMAND_TOPIC) and not speed_list_without_preset_modes(
-        config.get(CONF_SPEED_LIST)
-    ):
-        raise ValueError("No valid speeds configured")
-    return config
-
-
 def valid_speed_range_configuration(config):
     """Validate that the fan speed_range configuration is valid, throws if it isn't."""
     if config.get(CONF_SPEED_RANGE_MIN) == 0:
@@ -135,94 +128,109 @@ def valid_preset_mode_configuration(config):
     return config
 
 
+_PLATFORM_SCHEMA_BASE = mqtt.MQTT_RW_PLATFORM_SCHEMA.extend(
+    {
+        vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
+        vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
+        vol.Optional(CONF_COMMAND_TEMPLATE): cv.template,
+        vol.Optional(CONF_OSCILLATION_COMMAND_TOPIC): mqtt.valid_publish_topic,
+        vol.Optional(CONF_OSCILLATION_COMMAND_TEMPLATE): cv.template,
+        vol.Optional(CONF_OSCILLATION_STATE_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(CONF_OSCILLATION_VALUE_TEMPLATE): cv.template,
+        vol.Optional(CONF_PERCENTAGE_COMMAND_TOPIC): mqtt.valid_publish_topic,
+        vol.Optional(CONF_PERCENTAGE_COMMAND_TEMPLATE): cv.template,
+        vol.Optional(CONF_PERCENTAGE_STATE_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(CONF_PERCENTAGE_VALUE_TEMPLATE): cv.template,
+        # CONF_PRESET_MODE_COMMAND_TOPIC and CONF_PRESET_MODES_LIST must be used together
+        vol.Inclusive(
+            CONF_PRESET_MODE_COMMAND_TOPIC, "preset_modes"
+        ): mqtt.valid_publish_topic,
+        vol.Inclusive(
+            CONF_PRESET_MODES_LIST, "preset_modes", default=[]
+        ): cv.ensure_list,
+        vol.Optional(CONF_PRESET_MODE_COMMAND_TEMPLATE): cv.template,
+        vol.Optional(CONF_PRESET_MODE_STATE_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(CONF_PRESET_MODE_VALUE_TEMPLATE): cv.template,
+        vol.Optional(
+            CONF_SPEED_RANGE_MIN, default=DEFAULT_SPEED_RANGE_MIN
+        ): cv.positive_int,
+        vol.Optional(
+            CONF_SPEED_RANGE_MAX, default=DEFAULT_SPEED_RANGE_MAX
+        ): cv.positive_int,
+        vol.Optional(
+            CONF_PAYLOAD_RESET_PERCENTAGE, default=DEFAULT_PAYLOAD_RESET
+        ): cv.string,
+        vol.Optional(
+            CONF_PAYLOAD_RESET_PRESET_MODE, default=DEFAULT_PAYLOAD_RESET
+        ): cv.string,
+        vol.Optional(CONF_PAYLOAD_HIGH_SPEED, default=SPEED_HIGH): cv.string,
+        vol.Optional(CONF_PAYLOAD_LOW_SPEED, default=SPEED_LOW): cv.string,
+        vol.Optional(CONF_PAYLOAD_MEDIUM_SPEED, default=SPEED_MEDIUM): cv.string,
+        vol.Optional(CONF_PAYLOAD_OFF_SPEED, default=SPEED_OFF): cv.string,
+        vol.Optional(CONF_PAYLOAD_OFF, default=DEFAULT_PAYLOAD_OFF): cv.string,
+        vol.Optional(CONF_PAYLOAD_ON, default=DEFAULT_PAYLOAD_ON): cv.string,
+        vol.Optional(
+            CONF_PAYLOAD_OSCILLATION_OFF, default=OSCILLATE_OFF_PAYLOAD
+        ): cv.string,
+        vol.Optional(
+            CONF_PAYLOAD_OSCILLATION_ON, default=OSCILLATE_ON_PAYLOAD
+        ): cv.string,
+        vol.Optional(CONF_SPEED_COMMAND_TOPIC): mqtt.valid_publish_topic,
+        vol.Optional(
+            CONF_SPEED_LIST,
+            default=[SPEED_OFF, SPEED_LOW, SPEED_MEDIUM, SPEED_HIGH],
+        ): cv.ensure_list,
+        vol.Optional(CONF_SPEED_STATE_TOPIC): mqtt.valid_subscribe_topic,
+        vol.Optional(CONF_SPEED_VALUE_TEMPLATE): cv.template,
+        vol.Optional(CONF_STATE_VALUE_TEMPLATE): cv.template,
+    }
+).extend(MQTT_ENTITY_COMMON_SCHEMA.schema)
+
 PLATFORM_SCHEMA = vol.All(
+    _PLATFORM_SCHEMA_BASE,
+    valid_speed_range_configuration,
+    valid_preset_mode_configuration,
+)
+
+DISCOVERY_SCHEMA = vol.All(
     # CONF_SPEED_COMMAND_TOPIC, CONF_SPEED_LIST, CONF_SPEED_STATE_TOPIC, CONF_SPEED_VALUE_TEMPLATE and
     # Speeds SPEED_LOW, SPEED_MEDIUM, SPEED_HIGH SPEED_OFF,
-    # are deprecated, support will be removed after a quarter (2021.7)
-    cv.deprecated(CONF_PAYLOAD_HIGH_SPEED),
-    cv.deprecated(CONF_PAYLOAD_LOW_SPEED),
-    cv.deprecated(CONF_PAYLOAD_MEDIUM_SPEED),
-    cv.deprecated(CONF_SPEED_COMMAND_TOPIC),
-    cv.deprecated(CONF_SPEED_LIST),
-    cv.deprecated(CONF_SPEED_STATE_TOPIC),
-    cv.deprecated(CONF_SPEED_VALUE_TEMPLATE),
-    mqtt.MQTT_RW_PLATFORM_SCHEMA.extend(
-        {
-            vol.Optional(CONF_NAME, default=DEFAULT_NAME): cv.string,
-            vol.Optional(CONF_OPTIMISTIC, default=DEFAULT_OPTIMISTIC): cv.boolean,
-            vol.Optional(CONF_COMMAND_TEMPLATE): cv.template,
-            vol.Optional(CONF_OSCILLATION_COMMAND_TOPIC): mqtt.valid_publish_topic,
-            vol.Optional(CONF_OSCILLATION_COMMAND_TEMPLATE): cv.template,
-            vol.Optional(CONF_OSCILLATION_STATE_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(CONF_OSCILLATION_VALUE_TEMPLATE): cv.template,
-            vol.Optional(CONF_PERCENTAGE_COMMAND_TOPIC): mqtt.valid_publish_topic,
-            vol.Optional(CONF_PERCENTAGE_COMMAND_TEMPLATE): cv.template,
-            vol.Optional(CONF_PERCENTAGE_STATE_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(CONF_PERCENTAGE_VALUE_TEMPLATE): cv.template,
-            # CONF_PRESET_MODE_COMMAND_TOPIC and CONF_PRESET_MODES_LIST must be used together
-            vol.Inclusive(
-                CONF_PRESET_MODE_COMMAND_TOPIC, "preset_modes"
-            ): mqtt.valid_publish_topic,
-            vol.Inclusive(
-                CONF_PRESET_MODES_LIST, "preset_modes", default=[]
-            ): cv.ensure_list,
-            vol.Optional(CONF_PRESET_MODE_COMMAND_TEMPLATE): cv.template,
-            vol.Optional(CONF_PRESET_MODE_STATE_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(CONF_PRESET_MODE_VALUE_TEMPLATE): cv.template,
-            vol.Optional(
-                CONF_SPEED_RANGE_MIN, default=DEFAULT_SPEED_RANGE_MIN
-            ): cv.positive_int,
-            vol.Optional(
-                CONF_SPEED_RANGE_MAX, default=DEFAULT_SPEED_RANGE_MAX
-            ): cv.positive_int,
-            vol.Optional(
-                CONF_PAYLOAD_RESET_PERCENTAGE, default=DEFAULT_PAYLOAD_RESET
-            ): cv.string,
-            vol.Optional(
-                CONF_PAYLOAD_RESET_PRESET_MODE, default=DEFAULT_PAYLOAD_RESET
-            ): cv.string,
-            vol.Optional(CONF_PAYLOAD_HIGH_SPEED, default=SPEED_HIGH): cv.string,
-            vol.Optional(CONF_PAYLOAD_LOW_SPEED, default=SPEED_LOW): cv.string,
-            vol.Optional(CONF_PAYLOAD_MEDIUM_SPEED, default=SPEED_MEDIUM): cv.string,
-            vol.Optional(CONF_PAYLOAD_OFF_SPEED, default=SPEED_OFF): cv.string,
-            vol.Optional(CONF_PAYLOAD_OFF, default=DEFAULT_PAYLOAD_OFF): cv.string,
-            vol.Optional(CONF_PAYLOAD_ON, default=DEFAULT_PAYLOAD_ON): cv.string,
-            vol.Optional(
-                CONF_PAYLOAD_OSCILLATION_OFF, default=OSCILLATE_OFF_PAYLOAD
-            ): cv.string,
-            vol.Optional(
-                CONF_PAYLOAD_OSCILLATION_ON, default=OSCILLATE_ON_PAYLOAD
-            ): cv.string,
-            vol.Optional(CONF_SPEED_COMMAND_TOPIC): mqtt.valid_publish_topic,
-            vol.Optional(
-                CONF_SPEED_LIST,
-                default=[SPEED_OFF, SPEED_LOW, SPEED_MEDIUM, SPEED_HIGH],
-            ): cv.ensure_list,
-            vol.Optional(CONF_SPEED_STATE_TOPIC): mqtt.valid_subscribe_topic,
-            vol.Optional(CONF_SPEED_VALUE_TEMPLATE): cv.template,
-            vol.Optional(CONF_STATE_VALUE_TEMPLATE): cv.template,
-        }
-    ).extend(MQTT_ENTITY_COMMON_SCHEMA.schema),
-    valid_fan_speed_configuration,
+    # are no longer supported, support was removed in release 2021.12
+    cv.removed(CONF_PAYLOAD_HIGH_SPEED),
+    cv.removed(CONF_PAYLOAD_LOW_SPEED),
+    cv.removed(CONF_PAYLOAD_MEDIUM_SPEED),
+    cv.removed(CONF_SPEED_COMMAND_TOPIC),
+    cv.removed(CONF_SPEED_LIST),
+    cv.removed(CONF_SPEED_STATE_TOPIC),
+    cv.removed(CONF_SPEED_VALUE_TEMPLATE),
+    _PLATFORM_SCHEMA_BASE.extend({}, extra=vol.REMOVE_EXTRA),
     valid_speed_range_configuration,
     valid_preset_mode_configuration,
 )
 
 
 async def async_setup_platform(
-    hass: HomeAssistant, config: ConfigType, async_add_entities, discovery_info=None
-):
+    hass: HomeAssistant,
+    config: ConfigType,
+    async_add_entities: AddEntitiesCallback,
+    discovery_info: DiscoveryInfoType | None = None,
+) -> None:
     """Set up MQTT fan through configuration.yaml."""
-    await async_setup_reload_service(hass, DOMAIN, PLATFORMS)
-    await _async_setup_entity(hass, async_add_entities, config)
+    await async_setup_platform_helper(
+        hass, fan.DOMAIN, config, async_add_entities, _async_setup_entity
+    )
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up MQTT fan dynamically through MQTT discovery."""
     setup = functools.partial(
         _async_setup_entity, hass, async_add_entities, config_entry=config_entry
     )
-    await async_setup_entry_helper(hass, fan.DOMAIN, setup, PLATFORM_SCHEMA)
+    await async_setup_entry_helper(hass, fan.DOMAIN, setup, DISCOVERY_SCHEMA)
 
 
 async def _async_setup_entity(
@@ -235,13 +243,12 @@ async def _async_setup_entity(
 class MqttFan(MqttEntity, FanEntity):
     """A MQTT fan component."""
 
+    _entity_id_format = fan.ENTITY_ID_FORMAT
     _attributes_extra_blocked = MQTT_FAN_ATTRIBUTES_BLOCKED
 
     def __init__(self, hass, config, config_entry, discovery_data):
         """Initialize the MQTT fan."""
-        self._state = False
-        # self._speed will be removed after a quarter (2021.7)
-        self._speed = None
+        self._state = None
         self._percentage = None
         self._preset_mode = None
         self._oscillation = None
@@ -255,17 +262,13 @@ class MqttFan(MqttEntity, FanEntity):
         self._optimistic_oscillation = None
         self._optimistic_percentage = None
         self._optimistic_preset_mode = None
-        self._optimistic_speed = None
-
-        self._legacy_speeds_list = []
-        self._legacy_speeds_list_no_off = []
 
         MqttEntity.__init__(self, hass, config, config_entry, discovery_data)
 
     @staticmethod
     def config_schema():
         """Return the config schema."""
-        return PLATFORM_SCHEMA
+        return DISCOVERY_SCHEMA
 
     def _setup_from_config(self, config):
         """(Re)Setup the entity."""
@@ -282,8 +285,6 @@ class MqttFan(MqttEntity, FanEntity):
                 CONF_PERCENTAGE_COMMAND_TOPIC,
                 CONF_PRESET_MODE_STATE_TOPIC,
                 CONF_PRESET_MODE_COMMAND_TOPIC,
-                CONF_SPEED_STATE_TOPIC,
-                CONF_SPEED_COMMAND_TOPIC,
                 CONF_OSCILLATION_STATE_TOPIC,
                 CONF_OSCILLATION_COMMAND_TOPIC,
             )
@@ -292,8 +293,6 @@ class MqttFan(MqttEntity, FanEntity):
             CONF_STATE: config.get(CONF_STATE_VALUE_TEMPLATE),
             ATTR_PERCENTAGE: config.get(CONF_PERCENTAGE_VALUE_TEMPLATE),
             ATTR_PRESET_MODE: config.get(CONF_PRESET_MODE_VALUE_TEMPLATE),
-            # ATTR_SPEED is deprecated in the schema, support will be removed after a quarter (2021.7)
-            ATTR_SPEED: config.get(CONF_SPEED_VALUE_TEMPLATE),
             ATTR_OSCILLATING: config.get(CONF_OSCILLATION_VALUE_TEMPLATE),
         }
         self._command_templates = {
@@ -307,21 +306,9 @@ class MqttFan(MqttEntity, FanEntity):
             "STATE_OFF": config[CONF_PAYLOAD_OFF],
             "OSCILLATE_ON_PAYLOAD": config[CONF_PAYLOAD_OSCILLATION_ON],
             "OSCILLATE_OFF_PAYLOAD": config[CONF_PAYLOAD_OSCILLATION_OFF],
-            # The use of legacy speeds is deprecated in the schema, support will be removed after a quarter (2021.7)
-            "SPEED_LOW": config[CONF_PAYLOAD_LOW_SPEED],
-            "SPEED_MEDIUM": config[CONF_PAYLOAD_MEDIUM_SPEED],
-            "SPEED_HIGH": config[CONF_PAYLOAD_HIGH_SPEED],
-            "SPEED_OFF": config[CONF_PAYLOAD_OFF_SPEED],
             "PERCENTAGE_RESET": config[CONF_PAYLOAD_RESET_PERCENTAGE],
             "PRESET_MODE_RESET": config[CONF_PAYLOAD_RESET_PRESET_MODE],
         }
-        # The use of legacy speeds is deprecated in the schema, support will be removed after a quarter (2021.7)
-        self._feature_legacy_speeds = not self._topic[CONF_SPEED_COMMAND_TOPIC] is None
-        if self._feature_legacy_speeds:
-            self._legacy_speeds_list = config[CONF_SPEED_LIST]
-            self._legacy_speeds_list_no_off = speed_list_without_preset_modes(
-                self._legacy_speeds_list
-            )
 
         self._feature_percentage = CONF_PERCENTAGE_COMMAND_TOPIC in config
         self._feature_preset_mode = CONF_PRESET_MODE_COMMAND_TOPIC in config
@@ -330,10 +317,11 @@ class MqttFan(MqttEntity, FanEntity):
         else:
             self._preset_modes = []
 
-        if self._feature_percentage:
-            self._speed_count = min(int_states_in_range(self._speed_range), 100)
-        else:
-            self._speed_count = len(self._legacy_speeds_list_no_off) or 100
+        self._speed_count = (
+            min(int_states_in_range(self._speed_range), 100)
+            if self._feature_percentage
+            else 100
+        )
 
         optimistic = config[CONF_OPTIMISTIC]
         self._optimistic = optimistic or self._topic[CONF_STATE_TOPIC] is None
@@ -346,29 +334,29 @@ class MqttFan(MqttEntity, FanEntity):
         self._optimistic_preset_mode = (
             optimistic or self._topic[CONF_PRESET_MODE_STATE_TOPIC] is None
         )
-        self._optimistic_speed = (
-            optimistic or self._topic[CONF_SPEED_STATE_TOPIC] is None
-        )
 
         self._supported_features = 0
         self._supported_features |= (
             self._topic[CONF_OSCILLATION_COMMAND_TOPIC] is not None
             and SUPPORT_OSCILLATE
         )
-        if self._feature_percentage or self._feature_legacy_speeds:
+        if self._feature_percentage:
             self._supported_features |= SUPPORT_SET_SPEED
         if self._feature_preset_mode:
             self._supported_features |= SUPPORT_PRESET_MODE
 
-        for tpl_dict in [self._command_templates, self._value_templates]:
-            for key, tpl in tpl_dict.items():
-                if tpl is None:
-                    tpl_dict[key] = lambda value: value
-                else:
-                    tpl.hass = self.hass
-                    tpl_dict[key] = tpl.async_render_with_possible_json_value
+        for key, tpl in self._command_templates.items():
+            self._command_templates[key] = MqttCommandTemplate(
+                tpl, entity=self
+            ).async_render
 
-    async def _subscribe_topics(self):  # noqa: C901
+        for key, tpl in self._value_templates.items():
+            self._value_templates[key] = MqttValueTemplate(
+                tpl,
+                entity=self,
+            ).async_render_with_possible_json_value
+
+    def _prepare_subscribe_topics(self):
         """(Re)Subscribe to topics."""
         topics = {}
 
@@ -384,6 +372,8 @@ class MqttFan(MqttEntity, FanEntity):
                 self._state = True
             elif payload == self._payload["STATE_OFF"]:
                 self._state = False
+            elif payload == PAYLOAD_NONE:
+                self._state = None
             self.async_write_ha_state()
 
         if self._topic[CONF_STATE_TOPIC] is not None:
@@ -391,6 +381,7 @@ class MqttFan(MqttEntity, FanEntity):
                 "topic": self._topic[CONF_STATE_TOPIC],
                 "msg_callback": state_received,
                 "qos": self._config[CONF_QOS],
+                "encoding": self._config[CONF_ENCODING] or None,
             }
 
         @callback
@@ -405,7 +396,6 @@ class MqttFan(MqttEntity, FanEntity):
                 return
             if rendered_percentage_payload == self._payload["PERCENTAGE_RESET"]:
                 self._percentage = None
-                self._speed = None
                 self.async_write_ha_state()
                 return
             try:
@@ -436,6 +426,7 @@ class MqttFan(MqttEntity, FanEntity):
                 "topic": self._topic[CONF_PERCENTAGE_STATE_TOPIC],
                 "msg_callback": percentage_received,
                 "qos": self._config[CONF_QOS],
+                "encoding": self._config[CONF_ENCODING] or None,
             }
             self._percentage = None
 
@@ -468,54 +459,10 @@ class MqttFan(MqttEntity, FanEntity):
                 "topic": self._topic[CONF_PRESET_MODE_STATE_TOPIC],
                 "msg_callback": preset_mode_received,
                 "qos": self._config[CONF_QOS],
+                "encoding": self._config[CONF_ENCODING] or None,
             }
             self._preset_mode = None
 
-        # The use of legacy speeds is deprecated in the schema, support will be removed after a quarter (2021.7)
-        @callback
-        @log_messages(self.hass, self.entity_id)
-        def speed_received(msg):
-            """Handle new received MQTT message for the speed."""
-            speed_payload = self._value_templates[ATTR_SPEED](msg.payload)
-            if speed_payload == self._payload["SPEED_LOW"]:
-                speed = SPEED_LOW
-            elif speed_payload == self._payload["SPEED_MEDIUM"]:
-                speed = SPEED_MEDIUM
-            elif speed_payload == self._payload["SPEED_HIGH"]:
-                speed = SPEED_HIGH
-            elif speed_payload == self._payload["SPEED_OFF"]:
-                speed = SPEED_OFF
-            else:
-                speed = None
-
-            if speed and speed in self._legacy_speeds_list:
-                self._speed = speed
-            else:
-                _LOGGER.warning(
-                    "'%s' received on topic %s. '%s' is not a valid speed",
-                    msg.payload,
-                    msg.topic,
-                    speed,
-                )
-                return
-
-            if speed in self._legacy_speeds_list_no_off:
-                self._percentage = ordered_list_item_to_percentage(
-                    self._legacy_speeds_list_no_off, speed
-                )
-            elif speed == SPEED_OFF:
-                self._percentage = 0
-
-            self.async_write_ha_state()
-
-        if self._topic[CONF_SPEED_STATE_TOPIC] is not None:
-            topics[CONF_SPEED_STATE_TOPIC] = {
-                "topic": self._topic[CONF_SPEED_STATE_TOPIC],
-                "msg_callback": speed_received,
-                "qos": self._config[CONF_QOS],
-            }
-            self._speed = SPEED_OFF
-
         @callback
         @log_messages(self.hass, self.entity_id)
         def oscillation_received(msg):
@@ -535,29 +482,28 @@ class MqttFan(MqttEntity, FanEntity):
                 "topic": self._topic[CONF_OSCILLATION_STATE_TOPIC],
                 "msg_callback": oscillation_received,
                 "qos": self._config[CONF_QOS],
+                "encoding": self._config[CONF_ENCODING] or None,
             }
             self._oscillation = False
 
-        self._sub_state = await subscription.async_subscribe_topics(
+        self._sub_state = subscription.async_prepare_subscribe_topics(
             self.hass, self._sub_state, topics
         )
 
+    async def _subscribe_topics(self):
+        """(Re)Subscribe to topics."""
+        await subscription.async_subscribe_topics(self.hass, self._sub_state)
+
     @property
     def assumed_state(self):
         """Return true if we do optimistic updates."""
         return self._optimistic
 
     @property
-    def is_on(self):
+    def is_on(self) -> bool | None:
         """Return true if device is on."""
         return self._state
 
-    # The use of legacy speeds is deprecated in the schema, support will be removed after a quarter (2021.7)
-    @property
-    def _implemented_speed(self) -> bool:
-        """Return true if speed has been implemented."""
-        return self._feature_legacy_speeds
-
     @property
     def percentage(self):
         """Return the current percentage."""
@@ -573,22 +519,11 @@ class MqttFan(MqttEntity, FanEntity):
         """Get the list of available preset modes."""
         return self._preset_modes
 
-    # The speed_list property is deprecated in the schema, support will be removed after a quarter (2021.7)
-    @property
-    def speed_list(self) -> list:
-        """Get the list of available speeds."""
-        return self._legacy_speeds_list_no_off
-
     @property
     def supported_features(self) -> int:
         """Flag supported features."""
         return self._supported_features
 
-    @property
-    def speed(self):
-        """Return the current speed."""
-        return self._speed
-
     @property
     def speed_count(self) -> int:
         """Return the number of speeds the fan supports."""
@@ -612,20 +547,17 @@ class MqttFan(MqttEntity, FanEntity):
         This method is a coroutine.
         """
         mqtt_payload = self._command_templates[CONF_STATE](self._payload["STATE_ON"])
-        mqtt.async_publish(
-            self.hass,
+        await self.async_publish(
             self._topic[CONF_COMMAND_TOPIC],
             mqtt_payload,
             self._config[CONF_QOS],
             self._config[CONF_RETAIN],
+            self._config[CONF_ENCODING],
         )
         if percentage:
             await self.async_set_percentage(percentage)
         if preset_mode:
             await self.async_set_preset_mode(preset_mode)
-        # The speed attribute deprecated in the schema, support will be removed after a quarter (2021.7)
-        if speed and not percentage and not preset_mode:
-            await self.async_set_speed(speed)
         if self._optimistic:
             self._state = True
             self.async_write_ha_state()
@@ -636,12 +568,12 @@ class MqttFan(MqttEntity, FanEntity):
         This method is a coroutine.
         """
         mqtt_payload = self._command_templates[CONF_STATE](self._payload["STATE_OFF"])
-        mqtt.async_publish(
-            self.hass,
+        await self.async_publish(
             self._topic[CONF_COMMAND_TOPIC],
             mqtt_payload,
             self._config[CONF_QOS],
             self._config[CONF_RETAIN],
+            self._config[CONF_ENCODING],
         )
         if self._optimistic:
             self._state = False
@@ -656,26 +588,13 @@ class MqttFan(MqttEntity, FanEntity):
             percentage_to_ranged_value(self._speed_range, percentage)
         )
         mqtt_payload = self._command_templates[ATTR_PERCENTAGE](percentage_payload)
-        # Legacy are deprecated in the schema, support will be removed after a quarter (2021.7)
-        if self._feature_legacy_speeds:
-            if percentage:
-                await self.async_set_speed(
-                    percentage_to_ordered_list_item(
-                        self._legacy_speeds_list_no_off,
-                        percentage,
-                    )
-                )
-            elif SPEED_OFF in self._legacy_speeds_list:
-                await self.async_set_speed(SPEED_OFF)
-
-        if self._feature_percentage:
-            mqtt.async_publish(
-                self.hass,
-                self._topic[CONF_PERCENTAGE_COMMAND_TOPIC],
-                mqtt_payload,
-                self._config[CONF_QOS],
-                self._config[CONF_RETAIN],
-            )
+        await self.async_publish(
+            self._topic[CONF_PERCENTAGE_COMMAND_TOPIC],
+            mqtt_payload,
+            self._config[CONF_QOS],
+            self._config[CONF_RETAIN],
+            self._config[CONF_ENCODING],
+        )
 
         if self._optimistic_percentage:
             self._percentage = percentage
@@ -692,51 +611,18 @@ class MqttFan(MqttEntity, FanEntity):
 
         mqtt_payload = self._command_templates[ATTR_PRESET_MODE](preset_mode)
 
-        mqtt.async_publish(
-            self.hass,
+        await self.async_publish(
             self._topic[CONF_PRESET_MODE_COMMAND_TOPIC],
             mqtt_payload,
             self._config[CONF_QOS],
             self._config[CONF_RETAIN],
+            self._config[CONF_ENCODING],
         )
 
         if self._optimistic_preset_mode:
             self._preset_mode = preset_mode
             self.async_write_ha_state()
 
-    # async_set_speed is deprecated, support will be removed after a quarter (2021.7)
-    async def async_set_speed(self, speed: str) -> None:
-        """Set the speed of the fan.
-
-        This method is a coroutine.
-        """
-        speed_payload = None
-        if speed in self._legacy_speeds_list:
-            if speed == SPEED_LOW:
-                speed_payload = self._payload["SPEED_LOW"]
-            elif speed == SPEED_MEDIUM:
-                speed_payload = self._payload["SPEED_MEDIUM"]
-            elif speed == SPEED_HIGH:
-                speed_payload = self._payload["SPEED_HIGH"]
-            else:
-                speed_payload = self._payload["SPEED_OFF"]
-        else:
-            _LOGGER.warning("'%s' is not a valid speed", speed)
-            return
-
-        if speed_payload:
-            mqtt.async_publish(
-                self.hass,
-                self._topic[CONF_SPEED_COMMAND_TOPIC],
-                speed_payload,
-                self._config[CONF_QOS],
-                self._config[CONF_RETAIN],
-            )
-
-            if self._optimistic_speed and speed_payload:
-                self._speed = speed
-                self.async_write_ha_state()
-
     async def async_oscillate(self, oscillating: bool) -> None:
         """Set oscillation.
 
@@ -751,12 +637,12 @@ class MqttFan(MqttEntity, FanEntity):
                 self._payload["OSCILLATE_OFF_PAYLOAD"]
             )
 
-        mqtt.async_publish(
-            self.hass,
+        await self.async_publish(
             self._topic[CONF_OSCILLATION_COMMAND_TOPIC],
             mqtt_payload,
             self._config[CONF_QOS],
             self._config[CONF_RETAIN],
+            self._config[CONF_ENCODING],
         )
 
         if self._optimistic_oscillation:
