@@ -11,7 +11,7 @@ from tololib.errors import ResponseTimedOutError
 from tololib.message_info import SettingsInfo, StatusInfo
 
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import CONF_HOST
+from homeassistant.const import CONF_HOST, Platform
 from homeassistant.core import HomeAssistant
 from homeassistant.helpers.entity import DeviceInfo
 from homeassistant.helpers.update_coordinator import (
@@ -23,13 +23,14 @@ from homeassistant.helpers.update_coordinator import (
 from .const import DEFAULT_RETRY_COUNT, DEFAULT_RETRY_TIMEOUT, DOMAIN
 
 PLATFORMS = [
-    "binary_sensor",
-    "button",
-    "climate",
-    "fan",
-    "light",
-    "select",
-    "sensor",
+    Platform.BINARY_SENSOR,
+    Platform.BUTTON,
+    Platform.CLIMATE,
+    Platform.FAN,
+    Platform.LIGHT,
+    Platform.NUMBER,
+    Platform.SELECT,
+    Platform.SENSOR,
 ]
 
 _LOGGER = logging.getLogger(__name__)
