@@ -3,7 +3,7 @@ from __future__ import annotations
 
 from dataclasses import dataclass
 import json
-from typing import Any
+from typing import Any, cast
 
 from tuya_iot import TuyaDevice, TuyaDeviceManager
 
@@ -19,17 +19,39 @@ from homeassistant.components.light import (
     LightEntityDescription,
 )
 from homeassistant.config_entries import ConfigEntry
-from homeassistant.const import ENTITY_CATEGORY_CONFIG
 from homeassistant.core import HomeAssistant, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
+from homeassistant.helpers.entity import EntityCategory
 from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from . import HomeAssistantTuyaData
 from .base import IntegerTypeData, TuyaEntity
-from .const import DOMAIN, TUYA_DISCOVERY_NEW, DPCode, WorkMode
+from .const import DOMAIN, TUYA_DISCOVERY_NEW, DPCode, DPType, WorkMode
 from .util import remap_value
 
 
+@dataclass
+class ColorTypeData:
+    """Color Type Data."""
+
+    h_type: IntegerTypeData
+    s_type: IntegerTypeData
+    v_type: IntegerTypeData
+
+
+DEFAULT_COLOR_TYPE_DATA = ColorTypeData(
+    h_type=IntegerTypeData(DPCode.COLOUR_DATA_HSV, min=1, scale=0, max=360, step=1),
+    s_type=IntegerTypeData(DPCode.COLOUR_DATA_HSV, min=1, scale=0, max=255, step=1),
+    v_type=IntegerTypeData(DPCode.COLOUR_DATA_HSV, min=1, scale=0, max=255, step=1),
+)
+
+DEFAULT_COLOR_TYPE_DATA_V2 = ColorTypeData(
+    h_type=IntegerTypeData(DPCode.COLOUR_DATA_HSV, min=1, scale=0, max=360, step=1),
+    s_type=IntegerTypeData(DPCode.COLOUR_DATA_HSV, min=1, scale=0, max=1000, step=1),
+    v_type=IntegerTypeData(DPCode.COLOUR_DATA_HSV, min=1, scale=0, max=1000, step=1),
+)
+
+
 @dataclass
 class TuyaLightEntityDescription(LightEntityDescription):
     """Describe an Tuya light entity."""
@@ -40,9 +62,19 @@ class TuyaLightEntityDescription(LightEntityDescription):
     color_data: DPCode | tuple[DPCode, ...] | None = None
     color_mode: DPCode | None = None
     color_temp: DPCode | tuple[DPCode, ...] | None = None
+    default_color_type: ColorTypeData = DEFAULT_COLOR_TYPE_DATA
 
 
 LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
+    # Curtain Switch
+    # https://developer.tuya.com/en/docs/iot/category-clkg?id=Kaiuz0gitil39
+    "clkg": (
+        TuyaLightEntityDescription(
+            key=DPCode.SWITCH_BACKLIGHT,
+            name="Backlight",
+            entity_category=EntityCategory.CONFIG,
+        ),
+    ),
     # String Lights
     # https://developer.tuya.com/en/docs/iot/dc?id=Kaof7taxmvadu
     "dc": (
@@ -63,6 +95,7 @@ LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
             brightness=DPCode.BRIGHT_VALUE,
             color_temp=DPCode.TEMP_VALUE,
             color_data=DPCode.COLOUR_DATA,
+            default_color_type=DEFAULT_COLOR_TYPE_DATA_V2,
         ),
     ),
     # Light
@@ -75,6 +108,13 @@ LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
             color_temp=(DPCode.TEMP_VALUE_V2, DPCode.TEMP_VALUE),
             color_data=(DPCode.COLOUR_DATA_V2, DPCode.COLOUR_DATA),
         ),
+        # Not documented
+        # Based on multiple reports: manufacturer customized Dimmer 2 switches
+        TuyaLightEntityDescription(
+            key=DPCode.SWITCH_LED_1,
+            name="Light",
+            brightness=DPCode.BRIGHT_VALUE_1,
+        ),
     ),
     # Ceiling Fan Light
     # https://developer.tuya.com/en/docs/iot/fsd?id=Kaof8eiei4c2v
@@ -125,6 +165,7 @@ LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
         TuyaLightEntityDescription(
             key=DPCode.SWITCH_BACKLIGHT,
             name="Backlight",
+            entity_category=EntityCategory.CONFIG,
         ),
     ),
     # Air conditioner
@@ -133,6 +174,38 @@ LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
         TuyaLightEntityDescription(
             key=DPCode.LIGHT,
             name="Backlight",
+            entity_category=EntityCategory.CONFIG,
+        ),
+    ),
+    # Unknown light product
+    # Found as VECINO RGBW as provided by diagnostics
+    # Not documented
+    "mbd": (
+        TuyaLightEntityDescription(
+            key=DPCode.SWITCH_LED,
+            color_mode=DPCode.WORK_MODE,
+            brightness=DPCode.BRIGHT_VALUE,
+            color_data=DPCode.COLOUR_DATA,
+        ),
+    ),
+    # Unknown product with light capabilities
+    # Fond in some diffusers, plugs and PIR flood lights
+    # Not documented
+    "qjdcz": (
+        TuyaLightEntityDescription(
+            key=DPCode.SWITCH_LED,
+            color_mode=DPCode.WORK_MODE,
+            brightness=DPCode.BRIGHT_VALUE,
+            color_data=DPCode.COLOUR_DATA,
+        ),
+    ),
+    # Heater
+    # https://developer.tuya.com/en/docs/iot/categoryqn?id=Kaiuz18kih0sm
+    "qn": (
+        TuyaLightEntityDescription(
+            key=DPCode.LIGHT,
+            name="Backlight",
+            entity_category=EntityCategory.CONFIG,
         ),
     ),
     # Smart Camera
@@ -146,7 +219,7 @@ LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
         TuyaLightEntityDescription(
             key=DPCode.BASIC_INDICATOR,
             name="Indicator Light",
-            entity_category=ENTITY_CATEGORY_CONFIG,
+            entity_category=EntityCategory.CONFIG,
         ),
     ),
     # Dimmer Switch
@@ -231,6 +304,16 @@ LIGHTS: dict[str, tuple[TuyaLightEntityDescription, ...]] = {
             color_temp=DPCode.TEMP_CONTROLLER,
         ),
     ),
+    # Fan
+    # https://developer.tuya.com/en/docs/iot/categoryfs?id=Kaiuz1xweel1c
+    "fs": (
+        TuyaLightEntityDescription(
+            key=DPCode.LIGHT,
+            color_mode=DPCode.WORK_MODE,
+            brightness=DPCode.BRIGHT_VALUE,
+            color_temp=DPCode.TEMP_VALUE,
+        ),
+    ),
 }
 
 # Socket (duplicate of `kg`)
@@ -242,28 +325,6 @@ LIGHTS["cz"] = LIGHTS["kg"]
 LIGHTS["pc"] = LIGHTS["kg"]
 
 
-@dataclass
-class ColorTypeData:
-    """Color Type Data."""
-
-    h_type: IntegerTypeData
-    s_type: IntegerTypeData
-    v_type: IntegerTypeData
-
-
-DEFAULT_COLOR_TYPE_DATA = ColorTypeData(
-    h_type=IntegerTypeData(min=1, scale=0, max=360, step=1),
-    s_type=IntegerTypeData(min=1, scale=0, max=255, step=1),
-    v_type=IntegerTypeData(min=1, scale=0, max=255, step=1),
-)
-
-DEFAULT_COLOR_TYPE_DATA_V2 = ColorTypeData(
-    h_type=IntegerTypeData(min=1, scale=0, max=360, step=1),
-    s_type=IntegerTypeData(min=1, scale=0, max=1000, step=1),
-    v_type=IntegerTypeData(min=1, scale=0, max=1000, step=1),
-)
-
-
 @dataclass
 class ColorData:
     """Color Data."""
@@ -301,10 +362,7 @@ async def async_setup_entry(
             device = hass_data.device_manager.device_map[device_id]
             if descriptions := LIGHTS.get(device.category):
                 for description in descriptions:
-                    if (
-                        description.key in device.function
-                        or description.key in device.status
-                    ):
+                    if description.key in device.status:
                         entities.append(
                             TuyaLightEntity(
                                 device, hass_data.device_manager, description
@@ -324,15 +382,14 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
     """Tuya light device."""
 
     entity_description: TuyaLightEntityDescription
-    _brightness_dpcode: DPCode | None = None
-    _brightness_max_type: IntegerTypeData | None = None
-    _brightness_min_type: IntegerTypeData | None = None
-    _brightness_type: IntegerTypeData | None = None
+
+    _brightness_max: IntegerTypeData | None = None
+    _brightness_min: IntegerTypeData | None = None
+    _brightness: IntegerTypeData | None = None
     _color_data_dpcode: DPCode | None = None
     _color_data_type: ColorTypeData | None = None
-    _color_mode_dpcode: DPCode | None = None
-    _color_temp_dpcode: DPCode | None = None
-    _color_temp_type: IntegerTypeData | None = None
+    _color_mode: DPCode | None = None
+    _color_temp: IntegerTypeData | None = None
 
     def __init__(
         self,
@@ -346,106 +403,51 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
         self._attr_unique_id = f"{super().unique_id}{description.key}"
         self._attr_supported_color_modes = {COLOR_MODE_ONOFF}
 
-        # Determine brightness DPCodes
-        if (
-            isinstance(description.brightness, DPCode)
-            and description.brightness in device.function
-        ):
-            self._brightness_dpcode = description.brightness
-        elif isinstance(description.brightness, tuple):
-            self._brightness_dpcode = next(
-                (
-                    dpcode
-                    for dpcode in description.brightness
-                    if dpcode in device.function
-                ),
-                None,
-            )
+        # Determine DPCodes
+        self._color_mode_dpcode = self.find_dpcode(
+            description.color_mode, prefer_function=True
+        )
 
-        # Determine color mode DPCode
-        if (
-            description.color_mode is not None
-            and description.color_mode in device.function
+        if int_type := self.find_dpcode(
+            description.brightness, dptype=DPType.INTEGER, prefer_function=True
         ):
-            self._color_mode_dpcode = description.color_mode
-
-        # Determine DPCodes for color temperature
-        if (
-            isinstance(description.color_temp, DPCode)
-            and description.color_temp in device.function
-        ):
-            self._color_temp_dpcode = description.color_temp
-        elif isinstance(description.color_temp, tuple):
-            self._color_temp_dpcode = next(
-                (
-                    dpcode
-                    for dpcode in description.color_temp
-                    if dpcode in device.function
-                ),
-                None,
-            )
-
-        # Determine DPCodes for color data
-        if (
-            isinstance(description.color_data, DPCode)
-            and description.color_data in device.function
-        ):
-            self._color_data_dpcode = description.color_data
-        elif isinstance(description.color_data, tuple):
-            self._color_data_dpcode = next(
-                (
-                    dpcode
-                    for dpcode in description.color_data
-                    if dpcode in device.function
-                ),
-                None,
-            )
-
-        # Update internals based on found brightness dpcode
-        if self._brightness_dpcode:
+            self._brightness = int_type
             self._attr_supported_color_modes.add(COLOR_MODE_BRIGHTNESS)
-            self._brightness_type = IntegerTypeData.from_json(
-                device.function[self._brightness_dpcode].values
+            self._brightness_max = self.find_dpcode(
+                description.brightness_max, dptype=DPType.INTEGER
+            )
+            self._brightness_min = self.find_dpcode(
+                description.brightness_min, dptype=DPType.INTEGER
             )
 
-            # Check if min/max capable
-            if (
-                description.brightness_max is not None
-                and description.brightness_min is not None
-                and description.brightness_max in device.function
-                and description.brightness_min in device.function
-            ):
-                self._brightness_max_type = IntegerTypeData.from_json(
-                    device.function[description.brightness_max].values
-                )
-                self._brightness_min_type = IntegerTypeData.from_json(
-                    device.function[description.brightness_min].values
-                )
-
-        # Update internals based on found color temperature dpcode
-        if self._color_temp_dpcode:
+        if int_type := self.find_dpcode(
+            description.color_temp, dptype=DPType.INTEGER, prefer_function=True
+        ):
+            self._color_temp = int_type
             self._attr_supported_color_modes.add(COLOR_MODE_COLOR_TEMP)
-            self._color_temp_type = IntegerTypeData.from_json(
-                device.function[self._color_temp_dpcode].values
-            )
 
-        # Update internals based on found color data dpcode
-        if self._color_data_dpcode:
+        if (
+            dpcode := self.find_dpcode(description.color_data, prefer_function=True)
+        ) and self.get_dptype(dpcode) == DPType.JSON:
+            self._color_data_dpcode = dpcode
             self._attr_supported_color_modes.add(COLOR_MODE_HS)
+            if dpcode in self.device.function:
+                values = cast(str, self.device.function[dpcode].values)
+            else:
+                values = self.device.status_range[dpcode].values
+
             # Fetch color data type information
-            if function_data := json.loads(
-                self.device.function[self._color_data_dpcode].values
-            ):
+            if function_data := json.loads(values):
                 self._color_data_type = ColorTypeData(
-                    h_type=IntegerTypeData(**function_data["h"]),
-                    s_type=IntegerTypeData(**function_data["s"]),
-                    v_type=IntegerTypeData(**function_data["v"]),
+                    h_type=IntegerTypeData(dpcode, **function_data["h"]),
+                    s_type=IntegerTypeData(dpcode, **function_data["s"]),
+                    v_type=IntegerTypeData(dpcode, **function_data["v"]),
                 )
             else:
                 # If no type is found, use a default one
-                self._color_data_type = DEFAULT_COLOR_TYPE_DATA
+                self._color_data_type = self.entity_description.default_color_type
                 if self._color_data_dpcode == DPCode.COLOUR_DATA_V2 or (
-                    self._brightness_type and self._brightness_type.max > 255
+                    self._brightness and self._brightness.max > 255
                 ):
                     self._color_data_type = DEFAULT_COLOR_TYPE_DATA_V2
 
@@ -458,7 +460,7 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
         """Turn on or control the light."""
         commands = [{"code": self.entity_description.key, "value": True}]
 
-        if self._color_temp_type and ATTR_COLOR_TEMP in kwargs:
+        if self._color_temp and ATTR_COLOR_TEMP in kwargs:
             if self._color_mode_dpcode:
                 commands += [
                     {
@@ -469,9 +471,9 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
 
             commands += [
                 {
-                    "code": self._color_temp_dpcode,
+                    "code": self._color_temp.dpcode,
                     "value": round(
-                        self._color_temp_type.remap_value_from(
+                        self._color_temp.remap_value_from(
                             kwargs[ATTR_COLOR_TEMP],
                             self.min_mireds,
                             self.max_mireds,
@@ -526,37 +528,31 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
         if (
             ATTR_BRIGHTNESS in kwargs
             and self.color_mode != COLOR_MODE_HS
-            and self._brightness_type
+            and self._brightness
         ):
             brightness = kwargs[ATTR_BRIGHTNESS]
 
             # If there is a min/max value, the brightness is actually limited.
             # Meaning it is actually not on a 0-255 scale.
             if (
-                self._brightness_max_type is not None
-                and self._brightness_min_type is not None
-                and self.entity_description.brightness_max is not None
-                and self.entity_description.brightness_min is not None
+                self._brightness_max is not None
+                and self._brightness_min is not None
                 and (
                     brightness_max := self.device.status.get(
-                        self.entity_description.brightness_max
+                        self._brightness_max.dpcode
                     )
                 )
                 is not None
                 and (
                     brightness_min := self.device.status.get(
-                        self.entity_description.brightness_min
+                        self._brightness_min.dpcode
                     )
                 )
                 is not None
             ):
                 # Remap values onto our scale
-                brightness_max = self._brightness_max_type.remap_value_to(
-                    brightness_max
-                )
-                brightness_min = self._brightness_min_type.remap_value_to(
-                    brightness_min
-                )
+                brightness_max = self._brightness_max.remap_value_to(brightness_max)
+                brightness_min = self._brightness_min.remap_value_to(brightness_min)
 
                 # Remap the brightness value from their min-max to our 0-255 scale
                 brightness = remap_value(
@@ -567,8 +563,8 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
 
             commands += [
                 {
-                    "code": self._brightness_dpcode,
-                    "value": round(self._brightness_type.remap_value_from(brightness)),
+                    "code": self._brightness.dpcode,
+                    "value": round(self._brightness.remap_value_from(brightness)),
                 },
             ]
 
@@ -585,39 +581,29 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
         if self.color_mode == COLOR_MODE_HS and (color_data := self._get_color_data()):
             return color_data.brightness
 
-        if not self._brightness_dpcode or not self._brightness_type:
+        if not self._brightness:
             return None
 
-        brightness = self.device.status.get(self._brightness_dpcode)
+        brightness = self.device.status.get(self._brightness.dpcode)
         if brightness is None:
             return None
 
         # Remap value to our scale
-        brightness = self._brightness_type.remap_value_to(brightness)
+        brightness = self._brightness.remap_value_to(brightness)
 
         # If there is a min/max value, the brightness is actually limited.
         # Meaning it is actually not on a 0-255 scale.
         if (
-            self._brightness_max_type is not None
-            and self._brightness_min_type is not None
-            and self.entity_description.brightness_max is not None
-            and self.entity_description.brightness_min is not None
-            and (
-                brightness_max := self.device.status.get(
-                    self.entity_description.brightness_max
-                )
-            )
+            self._brightness_max is not None
+            and self._brightness_min is not None
+            and (brightness_max := self.device.status.get(self._brightness_max.dpcode))
             is not None
-            and (
-                brightness_min := self.device.status.get(
-                    self.entity_description.brightness_min
-                )
-            )
+            and (brightness_min := self.device.status.get(self._brightness_min.dpcode))
             is not None
         ):
             # Remap values onto our scale
-            brightness_max = self._brightness_max_type.remap_value_to(brightness_max)
-            brightness_min = self._brightness_min_type.remap_value_to(brightness_min)
+            brightness_max = self._brightness_max.remap_value_to(brightness_max)
+            brightness_min = self._brightness_min.remap_value_to(brightness_min)
 
             # Remap the brightness value from their min-max to our 0-255 scale
             brightness = remap_value(
@@ -631,15 +617,15 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
     @property
     def color_temp(self) -> int | None:
         """Return the color_temp of the light."""
-        if not self._color_temp_dpcode or not self._color_temp_type:
+        if not self._color_temp:
             return None
 
-        temperature = self.device.status.get(self._color_temp_dpcode)
+        temperature = self.device.status.get(self._color_temp.dpcode)
         if temperature is None:
             return None
 
         return round(
-            self._color_temp_type.remap_value_to(
+            self._color_temp.remap_value_to(
                 temperature, self.min_mireds, self.max_mireds, reverse=True
             )
         )
@@ -663,9 +649,9 @@ class TuyaLightEntity(TuyaEntity, LightEntity):
             and self.device.status.get(self._color_mode_dpcode) != WorkMode.WHITE
         ):
             return COLOR_MODE_HS
-        if self._color_temp_dpcode:
+        if self._color_temp:
             return COLOR_MODE_COLOR_TEMP
-        if self._brightness_dpcode:
+        if self._brightness:
             return COLOR_MODE_BRIGHTNESS
         return COLOR_MODE_ONOFF
 
