@@ -3,7 +3,6 @@ from __future__ import annotations
 
 import asyncio
 from dataclasses import dataclass
-from enum import Enum
 from functools import partial
 import logging
 
@@ -12,10 +11,11 @@ from miio.powerstrip import PowerMode
 import voluptuous as vol
 
 from homeassistant.components.switch import (
-    DEVICE_CLASS_SWITCH,
+    SwitchDeviceClass,
     SwitchEntity,
     SwitchEntityDescription,
 )
+from homeassistant.config_entries import ConfigEntry
 from homeassistant.const import (
     ATTR_ENTITY_ID,
     ATTR_MODE,
@@ -23,8 +23,10 @@ from homeassistant.const import (
     CONF_HOST,
     CONF_TOKEN,
 )
-from homeassistant.core import callback
+from homeassistant.core import HomeAssistant, ServiceCall, callback
 import homeassistant.helpers.config_validation as cv
+from homeassistant.helpers.entity import EntityCategory
+from homeassistant.helpers.entity_platform import AddEntitiesCallback
 
 from .const import (
     CONF_DEVICE,
@@ -33,6 +35,8 @@ from .const import (
     CONF_MODEL,
     DOMAIN,
     FEATURE_FLAGS_AIRFRESH,
+    FEATURE_FLAGS_AIRFRESH_A1,
+    FEATURE_FLAGS_AIRFRESH_T2017,
     FEATURE_FLAGS_AIRHUMIDIFIER,
     FEATURE_FLAGS_AIRHUMIDIFIER_CA4,
     FEATURE_FLAGS_AIRHUMIDIFIER_CA_AND_CB,
@@ -61,6 +65,8 @@ from .const import (
     FEATURE_SET_LED,
     KEY_COORDINATOR,
     KEY_DEVICE,
+    MODEL_AIRFRESH_A1,
+    MODEL_AIRFRESH_T2017,
     MODEL_AIRFRESH_VA2,
     MODEL_AIRHUMIDIFIER_CA1,
     MODEL_AIRHUMIDIFIER_CA4,
@@ -165,7 +171,9 @@ SERVICE_TO_METHOD = {
 }
 
 MODEL_TO_FEATURES_MAP = {
+    MODEL_AIRFRESH_A1: FEATURE_FLAGS_AIRFRESH_A1,
     MODEL_AIRFRESH_VA2: FEATURE_FLAGS_AIRFRESH,
+    MODEL_AIRFRESH_T2017: FEATURE_FLAGS_AIRFRESH_T2017,
     MODEL_AIRHUMIDIFIER_CA1: FEATURE_FLAGS_AIRHUMIDIFIER_CA_AND_CB,
     MODEL_AIRHUMIDIFIER_CA4: FEATURE_FLAGS_AIRHUMIDIFIER_CA4,
     MODEL_AIRHUMIDIFIER_CB1: FEATURE_FLAGS_AIRHUMIDIFIER_CA_AND_CB,
@@ -206,6 +214,7 @@ SWITCH_TYPES = (
         icon="mdi:volume-high",
         method_on="async_set_buzzer_on",
         method_off="async_set_buzzer_off",
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_CHILD_LOCK,
@@ -214,6 +223,7 @@ SWITCH_TYPES = (
         icon="mdi:lock",
         method_on="async_set_child_lock_on",
         method_off="async_set_child_lock_off",
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_DRY,
@@ -222,6 +232,7 @@ SWITCH_TYPES = (
         icon="mdi:hair-dryer",
         method_on="async_set_dry_on",
         method_off="async_set_dry_off",
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_CLEAN,
@@ -231,6 +242,7 @@ SWITCH_TYPES = (
         method_on="async_set_clean_on",
         method_off="async_set_clean_off",
         available_with_device_off=False,
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_LED,
@@ -239,6 +251,7 @@ SWITCH_TYPES = (
         icon="mdi:led-outline",
         method_on="async_set_led_on",
         method_off="async_set_led_off",
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_LEARN_MODE,
@@ -247,6 +260,7 @@ SWITCH_TYPES = (
         icon="mdi:school-outline",
         method_on="async_set_learn_mode_on",
         method_off="async_set_learn_mode_off",
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_AUTO_DETECT,
@@ -254,6 +268,7 @@ SWITCH_TYPES = (
         name="Auto Detect",
         method_on="async_set_auto_detect_on",
         method_off="async_set_auto_detect_off",
+        entity_category=EntityCategory.CONFIG,
     ),
     XiaomiMiioSwitchDescription(
         key=ATTR_IONIZER,
@@ -262,11 +277,16 @@ SWITCH_TYPES = (
         icon="mdi:shimmer",
         method_on="async_set_ionizer_on",
         method_off="async_set_ionizer_off",
+        entity_category=EntityCategory.CONFIG,
     ),
 )
 
 
-async def async_setup_entry(hass, config_entry, async_add_entities):
+async def async_setup_entry(
+    hass: HomeAssistant,
+    config_entry: ConfigEntry,
+    async_add_entities: AddEntitiesCallback,
+) -> None:
     """Set up the switch from a config entry."""
     model = config_entry.data[CONF_MODEL]
     if model in (*MODELS_HUMIDIFIER, *MODELS_FAN):
@@ -397,7 +417,7 @@ async def async_setup_other_entry(hass, config_entry, async_add_entities):
                 model,
             )
 
-        async def async_service_handler(service):
+        async def async_service_handler(service: ServiceCall) -> None:
             """Map services to methods on XiaomiPlugGenericSwitch."""
             method = SERVICE_TO_METHOD.get(service.service)
             params = {
@@ -405,8 +425,7 @@ async def async_setup_other_entry(hass, config_entry, async_add_entities):
                 for key, value in service.data.items()
                 if key != ATTR_ENTITY_ID
             }
-            entity_ids = service.data.get(ATTR_ENTITY_ID)
-            if entity_ids:
+            if entity_ids := service.data.get(ATTR_ENTITY_ID):
                 devices = [
                     device
                     for device in hass.data[DATA_KEY].values()
@@ -466,14 +485,6 @@ class XiaomiGenericCoordinatedSwitch(XiaomiCoordinatedMiioEntity, SwitchEntity):
             return False
         return super().available
 
-    @staticmethod
-    def _extract_value_from_attribute(state, attribute):
-        value = getattr(state, attribute)
-        if isinstance(value, Enum):
-            return value.value
-
-        return value
-
     async def async_turn_on(self, **kwargs) -> None:
         """Turn on an option of the miio device."""
         method = getattr(self, self.entity_description.method_on)
@@ -622,7 +633,7 @@ class XiaomiGenericCoordinatedSwitch(XiaomiCoordinatedMiioEntity, SwitchEntity):
 class XiaomiGatewaySwitch(XiaomiGatewayDevice, SwitchEntity):
     """Representation of a XiaomiGatewaySwitch."""
 
-    _attr_device_class = DEVICE_CLASS_SWITCH
+    _attr_device_class = SwitchDeviceClass.SWITCH
 
     def __init__(self, coordinator, sub_device, entry, variable):
         """Initialize the XiaomiSensor."""
