@@ -1,4 +1,6 @@
 """Offer zone automation rules."""
+import logging
+
 import voluptuous as vol
 
 from homeassistant.const import (
@@ -8,9 +10,15 @@ from homeassistant.const import (
     CONF_PLATFORM,
     CONF_ZONE,
 )
-from homeassistant.core import CALLBACK_TYPE, HassJob, callback
-from homeassistant.helpers import condition, config_validation as cv, location
+from homeassistant.core import CALLBACK_TYPE, HassJob, HomeAssistant, callback
+from homeassistant.helpers import (
+    condition,
+    config_validation as cv,
+    entity_registry as er,
+    location,
+)
 from homeassistant.helpers.event import async_track_state_change_event
+from homeassistant.helpers.typing import ConfigType
 
 # mypy: allow-incomplete-defs, allow-untyped-defs
 # mypy: no-check-untyped-defs
@@ -19,12 +27,14 @@ EVENT_ENTER = "enter"
 EVENT_LEAVE = "leave"
 DEFAULT_EVENT = EVENT_ENTER
 
+_LOGGER = logging.getLogger(__name__)
+
 _EVENT_DESCRIPTION = {EVENT_ENTER: "entering", EVENT_LEAVE: "leaving"}
 
-TRIGGER_SCHEMA = vol.Schema(
+_TRIGGER_SCHEMA = cv.TRIGGER_BASE_SCHEMA.extend(
     {
         vol.Required(CONF_PLATFORM): "zone",
-        vol.Required(CONF_ENTITY_ID): cv.entity_ids,
+        vol.Required(CONF_ENTITY_ID): cv.entity_ids_or_uuids,
         vol.Required(CONF_ZONE): cv.entity_id,
         vol.Required(CONF_EVENT, default=DEFAULT_EVENT): vol.Any(
             EVENT_ENTER, EVENT_LEAVE
@@ -33,11 +43,23 @@ TRIGGER_SCHEMA = vol.Schema(
 )
 
 
+async def async_validate_trigger_config(
+    hass: HomeAssistant, config: ConfigType
+) -> ConfigType:
+    """Validate trigger config."""
+    config = _TRIGGER_SCHEMA(config)
+    registry = er.async_get(hass)
+    config[CONF_ENTITY_ID] = er.async_resolve_entity_ids(
+        registry, config[CONF_ENTITY_ID]
+    )
+    return config
+
+
 async def async_attach_trigger(
     hass, config, action, automation_info, *, platform_type: str = "zone"
 ) -> CALLBACK_TYPE:
     """Listen for state changes based on configuration."""
-    trigger_id = automation_info.get("trigger_id") if automation_info else None
+    trigger_data = automation_info["trigger_data"]
     entity_id = config.get(CONF_ENTITY_ID)
     zone_entity_id = config.get(CONF_ZONE)
     event = config.get(CONF_EVENT)
@@ -57,7 +79,14 @@ async def async_attach_trigger(
         ):
             return
 
-        zone_state = hass.states.get(zone_entity_id)
+        if not (zone_state := hass.states.get(zone_entity_id)):
+            _LOGGER.warning(
+                "Automation '%s' is referencing non-existing zone '%s' in a zone trigger",
+                automation_info["name"],
+                zone_entity_id,
+            )
+            return
+
         from_match = condition.zone(hass, zone_state, from_s) if from_s else False
         to_match = condition.zone(hass, zone_state, to_s) if to_s else False
 
@@ -74,6 +103,7 @@ async def async_attach_trigger(
                 job,
                 {
                     "trigger": {
+                        **trigger_data,
                         "platform": platform_type,
                         "entity_id": entity,
                         "from_state": from_s,
@@ -81,7 +111,6 @@ async def async_attach_trigger(
                         "zone": zone_state,
                         "event": event,
                         "description": description,
-                        "id": trigger_id,
                     }
                 },
                 to_s.context,
